/*
 *  Licensed to the Apache Software Foundation (ASF) under one or more
 *  contributor license agreements.  See the NOTICE file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.parsers;

import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;

import org.apache.commons.lang.StringUtils;

import fr.loof.fonzie.PersistenceQuery;
import fr.loof.fonzie.QueryParser;
import fr.loof.fonzie.keywords.Aggregate;

/**
 * @author <a href="nicolas@apache.org">Nicolas De Loof</a>
 */
public class NamedQueryParser
    implements QueryParser
{
    public PersistenceQuery parse( Class entityClass, Aggregate aggregate, String criteria, Class[] types )
    {
        criteria = StringUtils.uncapitalize( criteria );
        NamedQueries queries = (NamedQueries) entityClass.getAnnotation( NamedQueries.class );
        NamedQuery[] namedQueries;
        if ( queries != null )
        {
            namedQueries = queries.value();
        }
        else
        {
            NamedQuery namedQuery = (NamedQuery) entityClass.getAnnotation( NamedQuery.class );
            if ( namedQuery != null )
            {
                namedQueries = new NamedQuery[] { namedQuery };
            }
            else
            {
                return null;
            }
        }

        for ( NamedQuery namedQuery : namedQueries )
        {
            String name = namedQuery.name();
            int i = name.lastIndexOf( '.' );
            if ( i > 0 )
            {
                name = name.substring( i + 1 );
            }
            String uncapitalizedName = StringUtils.uncapitalize( name );
            if ( name.equals( criteria ) || uncapitalizedName.equals( criteria ) )
            {
                return new NamedPersistenceQuery( entityClass, namedQuery );
            }
        }
        return null;
    }
}
