/*
 *  Licensed to the Apache Softwimport org.apache.commons.lang.StringUtils;

import fr.loof.fonzie.JPQLPersistenceQuery;
file distributed with
 *  this work for additional information regarding copyright ownership.
 *  The ASF licenses this file to You under the Apache License, Version 2.0
 *  (the "License"); you may not use this file except in compliance with
 *  the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package fr.loof.fonzie.parsers;

import java.util.List;
import java.util.ListIterator;



public interface QueryElement
{
    /**
     * Post process the QueryElement list to check pre-conditions or add/change QueryElements. Typically some keyword
     * MAY rearrange the previous/next elements and/or replace them with extended elements.
     * 
     * @param elements List iterator on the queryElement list, pointing after current element
     * @return <code>false</code> if list post-processing must stop
     */
    public boolean postProcess( ListIterator<QueryElement> elements );

    /**
     * Process the JPQLPersistenceQuery to populate with JPQL.
     * 
     * @param jpqlQuery the Query beeing populated
     * @param params the Fonzie-method parameters types, mostly used for type compatibility check
     */
    void process( JPQLPersistenceQuery jpqlQuery, List<Class> params );
}